FlagHandler = {
	realm = "";
};

FlagList2 = { };



--[[

FlagHandler:SetRealmName(realm)

-- Stores the name of the realm currently being played on.  Should be called after log on.

]]--

function FlagHandler:SetRealm(realm)
	if ( realm == nil or realm == "" ) then
		return;
	end
	FlagRSP:printDebug("set realm: " .. realm)
	self.realm = realm
	
	if ( FlagList2[self.realm] == nil ) then
		FlagList2[self.realm] = { };
	end
end

--[[

FlagHandler:addFlag()

-- Add flag <flag> with value <value> for player <name>.

]]--
function FlagHandler:addFlag(flag, value, name, revision)
	if flag == nil then 
		return;
	end
    
    if revision == nil then
        revision = -1;
    end

	if ( flag == "Fullname" or flag == "Title" or flag == "FlagAddon" or flag == "Desc" or flag == "Surname" or flag == "Name" or flag == "Middle" ) then -- strings

		if FlagList2[self.realm] == nil then
			FlagList2[self.realm] = {};
		end
		
		if FlagList2[self.realm][name] == nil then
			FlagList2[self.realm][name] = {};
		end

		FlagList2[self.realm][name][flag] = value;
		FlagList2[self.realm][name].timeStamp = time();
		FlagList2[self.realm][name].faction = UnitFactionGroup("player");
        
        -- Saves the version of the field
        if ( FlagList2[self.realm][name].ver == nil ) then
            FlagList2[self.realm][name].ver = {};
        end
        FlagList2[self.realm][name].ver[flag] = revision; 
    elseif ( flag == "CharFlag" or flag == "RPFlag" ) then -- numbers, integer
        FlagList2[self.realm][name][flag] = tonumber(value);
        
        -- Version
        if ( FlagList2[self.realm][name].ver == nil ) then
            FlagList2[self.realm][name].ver = {};
        end
        FlagList2[self.realm][name].ver[flag] = revision;
	end 
end


--[[

FlagHandler:purgeFlags()

-- Purges old flags that haven't been seen for quite some time.

]]--
function FlagHandler:purgeFlags()

	-- oldest timestamp we accept.
	local delTime = time() - FlagRSPSettings.FlagPurgeInterval;
	local entriesDeleted = 0;
	local entriesDeletedRealm = 0;

	for realm, list in pairs(FlagList2) do
		-- e.g. list is FlagList["RPFlag"]
		-- e.g. flag is "RPFlag"
      
		entriesDeletedRealm = 0;

		for name, entry in pairs(list) do
			if entry.timeStamp == nil then
				list[name] = nil;
				entriesDeleted = entriesDeleted + 1;
				entriesDeletedRealm = entriesDeletedRealm + 1;
			elseif entry.timeStamp < delTime then
				list[name] = nil;
				entriesDeleted = entriesDeleted + 1;
				entriesDeletedRealm = entriesDeletedRealm + 1;
			end
		end

	end
   
	if entriesDeleted > 0 then
		local msg = string.gsub(FlagRSP_Locale_EntriesPurged, "%%s", entriesDeleted);
		FlagRSP:printM(msg);
	end
end


--[[

FlagHandler:purgeAllFlags()

-- Purges ALL saved flags.

]]--
function FlagHandler:purgeAllFlags()

	local entriesDeleted, flags = self:getStats();

	FlagList2 = nil;
	FlagList2 = {}; 
 
	local msg = string.gsub(FlagRSP_Locale_EntriesPurged, "%%s", entriesDeleted);
	FlagRSP:printM(msg);
end


--[[

FlagHandler:getFlag()

-- Get flag <flag> for player <name>.

]]--
function FlagHandler:getFlag(flag, name, realm)
	local value = "";
	if ( realm == nil ) then
		realm = self.realm;
	end

	if ( FlagList2[realm] ~= nil and FlagList2[realm][name] ~= nil and FlagList2[realm][name][flag] ~= nil ) then
		value = FlagList2[realm][name][flag];
	end

	if ( value == nil ) then
		value = "";
	end

	return value;
end

--[[

FlagHandler:getFlagRev()

-- Get revision of <flag> for player <name>.

]]--
function FlagHandler:getFlagRev(flag, name, realm)
	local rev = "";
	if ( realm == nil ) then
		realm = self.realm;
	end

	if ( FlagList2[realm] ~= nil and FlagList2[realm][name] ~= nil and FlagList2[realm][name][flag] ~= nil ) then
		rev = FlagList2[realm][name]["ver"][flag];
	end 

	if ( rev == nil ) then
		rev = "";
	end

	return rev;
end


--[[

- FlagHandler:CreateEntry(name)

-- If an entry at <name> doesn't exist, this will create the necessary fields for it.
-- It will not erase an old entry or any fields it does not have.

]]--

function FlagHandler:CreateEntry(name)
	if ( FlagList2[self.realm] == nil ) then
		FlagList2[self.realm] = {};
	end
	
	if ( FlagList2[self.realm][name] == nil ) then
		FlagList2[self.realm][name] = {};
	end
end

--[[

FlagHandler:IsEntry(name)

-- Returns whether or not a character has some form of flag.

]]--

function FlagHandler:IsEntry(name)
	if ( FlagList2[self.realm][name] ~= nil and FlagList2[self.realm][name]["Desc"] ~= nil ) then
		return true;
	else
		return false;
	end
end

--[[

FlagHandler:getStats()

-- Returns some statistics about saved flags.

]]--
function FlagHandler:getStats()
	local numEntries = 0;
	local flags = {};
	local numEntriesRealm = 0;

	for i = 0, 4 do
		flags[i] = 0;
	end

	for realm, list in pairs(FlagList2) do
		numEntriesRealm = 0;

		for name, entry in pairs(list) do
			numEntries = numEntries + 1;
			numEntriesRealm = numEntriesRealm + 1;
			
			if entry.RPFlag ~= nil then
				flags[entry.RPFlag] = flags[entry.RPFlag] + 1;
			end
		end
	end

	return numEntries, flags;
end

