
--[[ ==== Protocol handler =============================================================================== ]]--
FlagRSPProtocol = {
    timeForOnline = 300; -- 5 min
};

function FlagRSPProtocol:OnLoad()
    msp.my["VA"] = "flagRSP/"..FlagRSP.versionString or "";
    table.insert( msp.callback.received, FlagRSPProtocol_callbackMSP);
    msp:Update(); 
end

function FlagRSPProtocol:OnLogout()
    -- Saving msp data
    FlagRSPData.msp = {};
    FlagRSPData.msp.myver = {};
    FlagRSPData.msp.myver = msp.myver;
    FlagRSPData.msp.my = {};
    FlagRSPData.msp.my = msp.my;
end

-- Request
function FlagRSPProtocol:Request(unit,fields)  
    
    local request = false;
                
    if ( UnitExists(unit) and UnitPlayerControlled(unit) and not (UnitIsUnit(unit,"player") == 1)) then
        local unitName, unitRealm = UnitName(unit);  
        local reqName = "";
        if ( unitRealm == nil )then  
            reqName = unitName;
        elseif ( unitRealm ~= nil ) then
            reqName = unitName .. "-" .. unitRealm;
        end
        
        local fieldTable = FlagRSPProtocol.mspSupportedFields;
               
        request = msp:Request(reqName, fieldTable);
        
    end
    
    return request;
end

function FlagRSPProtocol:RequestDirect( name , fields )
    local request = false;
    -- local fieldTable = FlagRSPProtocol.mspSupportedFields;
           
    request = msp:Request(name, fields);
    return request;
end

-- Update Flag, Tooltip
function FlagRSPProtocol:Update(unitType, unit, request)
    if ( request ) then
        local unitName, unitRealm = UnitName(unit);
        local reqName = "";
        if ( unitType == "callback" ) then
            reqName = unit;
        elseif ( unitType == "isonline" ) then
            reqName = unit;
        elseif ( unitRealm ~= nil ) then  
            reqName = tostring(unitName) .. "-" .. tostring(unitRealm);
        else
            reqName = unitName;
        end
        FlagRSPProtocol:updateFlagList2( reqName, msp.char[reqName].field, msp.char[reqName].ver, "libmsp");
        FlagRSP:UpdateTooltip();
    end
end

-- UpdateOwn
function FlagRSPProtocol:UpdateOwn()    
    -- Update MSP Data
    -- msp.my.VA = "flagRSP/v".. tostring(GetAddOnMetadata("flagRSP |cFF029B78[MoP]|r", "Version"));
    -- msp.my.GU = UnitGUID("player");
    -- msp.my.GS = tostring( UnitSex("player"));
    -- msp.my.GC = select( 2, UnitClass("player"));
    -- msp.my.GR = select( 2, UnitRace("player"));
    msp.my["NA"] = FlagRSPConfigure:GetFullname();
    msp.my["NT"] = FlagRSPConfigure:GetTitle();
    msp.my["DE"] = FlagRSPConfigure:GetDescription();
    msp.my["FR"] = FlagRSPConfigure:GetRPFlag();
    msp.my["FC"] = FlagRSPConfigure:GetCharacterStatus();
    msp:Update();
    
    -- Update FlagList2
    local pName, _ = UnitName("player");
    FlagRSPProtocol:updateFlagList2( pName, msp.my, msp.myver, "libmspintern" ); 
    
    -- Update Tooltip, InfoBoxes
    FlagRSP:UpdateTooltip();
end

function FlagRSPProtocol:IsSupported( name )
    if ( msp.char[name] == nil ) then
        return false;
    end
    return msp.char[name].supported;
end

--[[ Checks if "name" is online, uses protocol for check
    Returns 
        - 1 if online
        - -1 if offline
        - 0 if offline and now information on this guy (maybe protocol not supported)
]]--
function FlagRSPProtocol:CheckIsOnline( name )
    local isOnline = 0;    
    
    if ( name ~= nil and FlagList2[FlagRSP.rName][name] ~= nil ) then
        local lastFlagTime = FlagHandler:getFlag("timeStamp", name, nil);
        local now = time();
        
        if ( lastFlagTime ~= nil ) then -- Entry exists
            local lastSeen = now - tonumber(lastFlagTime);
            if ( lastSeen <= self.timeForOnline ) then
                isOnline = 1;
            else
                isOnline = -1;
            end
        end
        if ( isOnline ~= 1 ) then -- Else ask for entry
            local request = FlagRSPProtocol:RequestDirect( name , self.mspSupportedFields );
            FlagRSPProtocol:Update("isonline", name, request)
        end       
    end
    
    
    return isOnline;
end


function FlagRSPProtocol:updateFlagList2( name, fields, revisions, protocol ) 
    if ( protocol == nil or protocol == "libmsp" or protocol == "libmspintern") then
        local c, r = fields, revisions;
        if (protocol == "libmsp") then
            -- Check if data supported
            if ( not FlagRSPProtocol:IsSupported( name ) ) then
                return;
            end
        end
        
        -- Full name
        if ( fields["NA"] ~= nil ) then
            FlagHandler:addFlag("Fullname", fields["NA"], name, revisions["NA"]);
        end
        
        -- Title
        if ( fields["NT"] ~= nil ) then
            FlagHandler:addFlag("Title", fields["NT"], name, revisions["NT"]);
        end   
        
        -- Flag RP
        if ( fields["FR"] ~= nil ) then
            FlagHandler:addFlag("RPFlag", tonumber(fields["FR"]), name, revisions["FR"]);
        end
            
        -- Flag Character
        if ( fields["FC"] ~= nil ) then        
            FlagHandler:addFlag("CharFlag", fields["FC"], name, revisions["FC"]);
        end
        
        -- Used Flagaddon
        if ( fields["VA"] ~= nil ) then
            FlagHandler:addFlag("FlagAddon", fields["VA"], name, revisions["VA"]);
            -- Check FlagRSP Version
            local name, ver,_ = strsplit("/",fields["VA"],3);
            if ( name == "flagRSP" ) then
                FlagRSP:checkVersion(ver);
            end
        end
        
        -- Desciption
        if ( fields["DE"] ~= nil ) then
            FlagHandler:addFlag("Desc", fields["DE"], name, revisions["DE"]);
        end
        
        -- Versions are updated in FlagHandler:addFlag   
    end
end


--[[ ==== Library support for LibMSP ===================================================================== ]]--
_G.msp_RPAddOn = "flagRSP";
FlagRSPProtocol.mspSupportedFields = { "NA", "NT", "FR", "FC", "VA", "DE" }; -- Tooltip
-- FlagRSPProtocol.mspSupportedIBFields = { "DE" }; -- InfoBox

function FlagRSPProtocol_callbackMSP( name )
    FlagRSPProtocol:Update("callback",name,true);
end

function FlagRSPProtocol_loadMsp()
    -- FlagRSPData must be available!
    if (FlagRSPData ~= nil ) then
        msp.my.NA = FlagRSPData.Fullname;
        msp.my.NT = FlagRSPData.Title;
        msp.my.DE = FlagRSPData.CharDesc;
        msp.my.FR = FlagRSPData.RPFlag;
        msp.my.FC = FlagRSPData.CharStatus;
            
        msp:Update();
        
        if(FlagRSPData.msp ~= nil ) then
            if ( FlagRSPData.msp.myver ~= nil ) then
                msp.myver = FlagRSPData.msp.myver;
            end
        end
    end
end